/*	Unterra.Monstoro.MapCompression library

	This library provides a means of generating "compressed" text representations of your
game's map levels. The exact amount of compression depends on the size and complexity
of the map, and the number of unique text representations you have specified. (It is possible
that complex maps will offer little or no compression.) You can also specify that certain
turf types should have clickable links on the map.


	To use it:

	* Define the UMMC_mapText var for your turfs. (Use only a single character.)
	This determines what the turfs will look like on the compressed map.
	It defaults to a space; you only need to override it for important turf types
	that you want to see on the map. For most games, you won't want to bother setting it
	for such common turfs as ground, rocks, and trees.

	* If you want any turfs to be clickable, set UMMC_mapLink to 1 for that turf type.

	* Create an instance of the UMMC_Controller object.

	* Whenever you want to display a map, call the controller's GetMap(mapLevel) proc to obtain
	an HTML text string that can be sent either to the text console or to a browser window.

	* If desired, use the controller's SetSpanDef and SetBodyDef procs to override the
	default text style.	(Be sure to use a monospace font, like Courier.) Note that some
	stylesheet commands work inconsistently between the text console and the browser, so you
	may need to experiment.


-- The compression process may distort distances considerably; however, directions will remain
roughly the same (and exactly the same when the direction is true north/south/east/west).

-- In general, the more distinct values of turf.UMMC_mapText you assign, the less
map compression will result. For example, when doors use the same mapText character as walls,
the map will usually be smaller than when doors have their own special character.

-- If you set UMMC_mapLink to 1 for a given object type, you probably want to make sure it
has a unique UMMC_mapText value. Otherwise, it is not guaranteed to show up on the map! Take a
look at the doors on map level 2 (the "HELLO" level)... since doors have the same mapText as
walls, they get compressed and don't show up.

-- Also be aware that even if a turf type does have a unique mapText value, it may be compressed
if it is part of a continuous row or column of three or more turfs of that type.

-- You can set the mapText and mapLink vars at runtime -- for example, if someone casts
a "secret door" spell and you're using a special character to indicate doors, you can change
the affected door's mapText to the wall character. If you make such changes at runtime, you
should re-generate that level's map, by calling the controller's GenerateMap(mapLevel) proc,
before again showing the map to a player.

*/


turf
	var
		UMMC_mapText = "&#160;" //nonbreaking space ("&nbsp;" doesn't seem to work well here)
		UMMC_mapLink


	Topic(href,href_list[])
		if(href_list["UMMC_click"]) UMMC_HandleMapClick(usr)
		. = ..()


	proc/UMMC_HandleMapClick(mob/clicker)
		//This is just a stub to be overridden as needed


UMMC_Controller
	var/list/maps[world.maxz]
	var
		spanDef; bodyDef
	var
		doCompression = 1


	New()
		SetSpanDef("font-family:Courier,monospace;text-align:center")
		SetBodyDef("")
		maps = new /list(world.maxz)


	proc/SetMaxZ(newMax)
		//Added 2003-0328 to support worlds that add Z levels at runtime.
		maps = new /list(newMax)


	proc/SetCompression(myValue)
		doCompression = myValue


	proc/SetSpanDef(myDef)
		spanDef = "<style>.UMMC{" + myDef + "}</style>"


	proc/SetBodyDef(myDef)
		bodyDef = "<style>BODY{" + myDef + "}</style>"


	proc/GetMap(curZ)
		if(!maps[curZ]) GenerateMap(curZ)
		return maps[curZ]


	proc/GenerateMap(curZ)
		var/list
			usefulX[world.maxx]
			usefulY[world.maxy]

		var/turf
			T; TPlus

		for(var/curX in 1 to world.maxx - 1)
			if(!doCompression)
				usefulX[curX] = 1
				if(curX > 1) continue

			for(var/curY in 1 to world.maxy - 1)
				if(!(usefulX[curX] && usefulY[curY])) T = locate(curX, curY, curZ)
				else continue

				if(!doCompression)
					usefulY[curY] = 1
					continue

				if(!usefulX[curX])
					TPlus = locate(curX + 1, curY, curZ)
					if(T.UMMC_mapText != TPlus.UMMC_mapText)
						usefulX[curX] = 1
						usefulX[curX + 1] = 1

				if(!usefulY[curY])
					TPlus = locate(curX, curY + 1, curZ)
					if(T.UMMC_mapText != TPlus.UMMC_mapText)
						usefulY[curY] = 1
						usefulY[curY + 1] = 1

		var/currentLine = bodyDef + spanDef + "<BODY><span class=\"UMMC\">"
		for(var/curY in world.maxy to 1 step -1)
			if(!usefulY[curY]) continue

			for(var/curX in 1 to world.maxx)
				if(!usefulX[curX]) continue

				T = locate(curX, curY, curZ)
				if(T.UMMC_mapLink) currentLine += \
					"<a href=\"?src=\ref[T];UMMC_click=1\">[T.UMMC_mapText]</a>"
				else currentLine += T.UMMC_mapText

			currentLine += "<br>"

		currentLine += "</span></BODY>"
		maps[curZ] = currentLine
